<?php

namespace LMS\Libraries\Injection;

/**
 * Provide functionality for removing hard-coded dependencies using pattern Dependency Injection
 *
 * @author  Andrei Nester
 */
class Container implements \ArrayAccess
{
    const MODE_SHARE = 0;
    const MODE_COPY = 1;

    private $services;
    private $factories;
    private $contextes;
    private $mode;

    public function __construct($mode = Container::MODE_SHARE)
    {
        $this->factories = array();
        $this->services = array();
        $this->contextes = array();
        $this->mode = $mode;
    }

    public function offsetSet($serviceName, $service)
    {
        if(is_callable($service)){
            $this->factories[$serviceName] = $service;
        }
        else{
            $this->services[$serviceName] = $service;
        }
    }

    public function offsetGet($serviceName)
    {
        if($this->mode === $this::MODE_SHARE){
            return $this->share($serviceName);
        }else{
            return $this->copy($serviceName);
        }
    }

    public function offsetExists($serviceName)
    {
        return array_key_exists($serviceName, $this->services) || array_key_exists($serviceName, $this->factories);
    }

    public function offsetUnset($serviceName)
    {
        unset($this->services[$serviceName]);
        unset($this->factories[$serviceName]);
    }

    public function setContext($serviceName, $context)
    {
        $this->contextes[$serviceName] = $context;
    }

    public function getContext($serviceName)
    {
        return array_key_exists($serviceName, $this->contextes) ? $this->contextes[$serviceName] : $this;
    }

    /**
     * Returns already created instance of service
     *
     * @param $serviceName
     * @return mixed
     * @throws \InvalidArgumentException
     */
    public function share($serviceName)
    {
        $services = $this->services;
        if(array_key_exists($serviceName, $services)){
            return $services[$serviceName];
        }
        $factories = $this->factories;
        if (!array_key_exists($serviceName, $factories)) {
            throw new \InvalidArgumentException(sprintf('Service and factory for service "%s" are not defined.', $serviceName));
        }

        $context = $this->getContext($serviceName);
        $services[$serviceName] = $factories[$serviceName]($context);
        return $services[$serviceName];
    }

    /**
     * Create new instance of service
     *
     * @param string $serviceName
     * @return mixed
     */
    public function copy($serviceName){

        $context = $this->getContext($serviceName);
        if (!isset($this->factories[$serviceName])) {
            throw new \InvalidArgumentException(sprintf('Factory for "%s" is not defined.', $serviceName));
        }

        return $this->factories[$serviceName]($context);
    }



}